#!/bin/sh
# Run this at the beginning of any script using PhdigetSBC config
# This fills in some environment variables which start with SBC_
# and provides functions to do common things


# config file locations, version
export SBC_USERSPACE=/mnt/userspace
export SBC_USERAPPS=$SBC_USERSPACE/userapps
export SBC_CONFIGDIR=$SBC_USERSPACE/.config
export SBC_CONFIGVERSION=1.0

# for development - this will be true is root is mounted over NFS
if grep "/dev/root / nfs" /proc/mounts >/dev/null; then
	export SBC_ROOTFS_NFS=true
else
	export SBC_ROOTFS_NFS=false
fi

# system
if [ -f $SBC_CONFIGDIR/system.conf ]; then
	. $SBC_CONFIGDIR/system.conf
	
	export SBC_HOSTNAME=$hostname
	export SBC_TZ=$posixtz
fi

# network
if [ -f $SBC_CONFIGDIR/network.conf ]; then
	. $SBC_CONFIGDIR/network.conf
	export SBC_SSHSTART=$sshstart

	export SBC_LANCONFIG=$lanconfig
	export SBC_IPADDR=$ipaddr
	export SBC_SUBNETMASK=$subnetmask
	export SBC_GATEWAY=$gateway

	export SBC_DNSCONFIG=$dnsconfig
	export SBC_DNSSERVER1=$dnsserver1
	export SBC_DNSSERVER2=$dnsserver2
	export SBC_DNSSERVER3=$dnsserver3
fi

# Phidget webservice
if [ -f $SBC_CONFIGDIR/phidgetwebservice21.conf ]; then
	. $SBC_CONFIGDIR/phidgetwebservice21.conf
	export SBC_PWSSTART=$pws_enabled
	export SBC_PWSPORT=$pws_port
	export SBC_PWSSERVERID="$pws_serverid"
	export SBC_PWSPASS=$pws_password
fi

# sets up DNS
do_dns()
{
	# make sure config is good
	if [ $SBC_DNSCONFIG = "manual" ]; then
		# at least one dns server needs to be defined
		if [ -z $SBC_DNSSERVER1 ]; then
			return 1
		fi
	else
		# Only other option is 'auto'
		if [ $SBC_DNSCONFIG != "auto" ]; then
			return 1
		fi
		# 'auto' requires lanconfig to be 'dhcp'
		if [ $SBC_LANCONFIG != "dhcp" ]; then
			return 1
		fi
	fi

	# set it up
	if [ $SBC_DNSCONFIG = "manual" ]; then
		echo "nameserver $SBC_DNSSERVER1" > /etc/resolv.conf
		[ -z $SBC_DNSSERVER2 ] || echo "nameserver $SBC_DNSSERVER2" >> /etc/resolv.conf
		[ -z $SBC_DNSSERVER3 ] || echo "nameserver $SBC_DNSSERVER3" >> /etc/resolv.conf
	fi
}

# creates interfaces file from info in network.conf
create_interfaces()
{
	# logic to make sure we have a valid configuration
	mkdir -p $SBC_CONFIGDIR/network

	if [ $SBC_LANCONFIG = "static" ]; then
		if [ -z $SBC_IPADDR ]; then
			return 1
		fi
		if [ -z $SBC_SUBNETMASK ]; then
			return 1
		fi
		if [ -z $SBC_GATEWAY ]; then
			return 1
		fi
		# dns has to be manual if using static IP
		if [ $SBC_DNSCONFIG != "manual" ]; then
			return 1
		fi
	else
		# only other option is dhcp
		if [ $SBC_LANCONFIG != "dhcp" ]; then
			return 1
		fi
	fi

	# write out to a new interfaces file

	cat > $SBC_CONFIGDIR/network/interfaces.new << EOF
# Configure Loopback
auto lo
iface lo inet loopback

auto eth0
EOF
	
	if [ $SBC_LANCONFIG = "static" ]; then
		cat >> $SBC_CONFIGDIR/network/interfaces.new << EOF
iface eth0 inet static
	address $SBC_IPADDR
	netmask $SBC_SUBNETMASK
	gateway $SBC_GATEWAY
EOF
	else
		echo "iface eth0 inet dhcp" >> $SBC_CONFIGDIR/network/interfaces.new
	fi

	# overwrite old interfaces file
	mv $SBC_CONFIGDIR/network/interfaces.new $SBC_CONFIGDIR/network/interfaces
}

# creates network.conf
create_network_config()
{
	cat > $SBC_CONFIGDIR/network.conf << EOF
sshstart="false"
lanconfig="dhcp"
ipaddr=""
subnetmask=""
gateway=""
dnsconfig="auto"
dnsserver1=""
dnsserver2=""
EOF
	. $SBC_CONFIGDIR/network.conf
	export SBC_SSHSTART=$sshstart

	export SBC_LANCONFIG=$lanconfig
	export SBC_IPADDR=$ipaddr
	export SBC_SUBNETMASK=$subnetmask
	export SBC_GATEWAY=$gateway

	export SBC_DNSCONFIG=$dnsconfig
	export SBC_DNSSERVER1=$dnsserver1
	export SBC_DNSSERVER2=$dnsserver2
	export SBC_DNSSERVER3=$dnsserver3
}

# creates wireless_network.conf
create_wireless_network_config()
{
	cat > $SBC_CONFIGDIR/wireless_network.conf << EOF
lanconfig="dhcp"
ipaddr=""
subnetmask=""
gateway=""
EOF
}

# creates phidgetwebservice21.conf
create_webservice_config()
{
	cat > $SBC_CONFIGDIR/phidgetwebservice21.conf << EOF
pws_enabled="true"
pws_port="5001"
pws_serverid=""
pws_password=""
EOF
	. $SBC_CONFIGDIR/phidgetwebservice21.conf
	export SBC_PWSSTART=$pws_enabled
	export SBC_PWSPORT=$pws_port
	export SBC_PWSSERVERID="$pws_serverid"
	export SBC_PWSPASS=$pws_password
}

# creates system.conf
create_system_config()
{
	cat > $SBC_CONFIGDIR/system.conf << EOF
hostname="$( hostname )"
posixtz="$( cat /etc/TZ )"
zoneinfo="-"
EOF
	. $SBC_CONFIGDIR/system.conf
	export SBC_HOSTNAME=$hostname
	export SBC_TZ=$posixtz
}

create_wpa_supplicant_config()
{
	cat > $SBC_CONFIGDIR/wpa_supplicant.conf << EOF
ctrl_interface=/var/run/wpa_supplicant
update_config=1
EOF
}

create_webcam_config()
{
	cat > $SBC_CONFIGDIR/webcam.conf << EOF
webcam_stream="false"
webcam_framerate="25"
webcam_res="320x240"
webcam_port="81"
webcam_pass=""
EOF
}

create_mdns_config()
{
	_version=$( grep "version" /etc/phidgetsbc_version | awk '{print $2}' )
	_board_version="$(cat /proc/cpuinfo | awk '/^Revision/ {print ("0x"$3)+0}')"
	_mac="$(/sbin/ifconfig eth0 | grep HWaddr | awk '{print $5}')"
	_hostname="$( hostname )"
	_board_name="$(cat /proc/cpuinfo | awk '/^Hardware/ {print $3}')"

	cat > $1 << EOF
PhidgetSBC Web Config ($_hostname) ($_mac)
_http._tcp.
80

PhidgetSBC ($_mac)
_phidget_sbc._tcp
80
txtvers=3
fversion=$_version
hversion=$_board_version
hostname=$_hostname
name=$_board_name
EOF
}

############################
# user app functions
############################

# $1 == app name
ua_make_config_file() {
	cat > $SBC_CONFIGDIR/ua_$1.conf << EOF
app_name="$1"
app_priority="20"
app_enabled="false"
app_daemonize="true"
app_cmd=""
app_args=""
EOF
}

# $1 == app name
ua_make_startup_file() {
	. $SBC_CONFIGDIR/ua_$1.conf
	cat > $SBC_CONFIGDIR/init.d/S$app_priority$1 << EOF
#!/bin/sh
#
# Start Custom app "$1"
#

# load config
. /etc/config.sh
. \$SBC_CONFIGDIR/ua_$1.conf

app_file="\$SBC_USERAPPS/\$app_name/\$app_cmd"

if [ "\${app_cmd##*\.}" = "class" ]; then
	run_app="jamvm \${app_cmd%\\.class}"
elif [ "\${app_cmd##*\.}" = "jar" ]; then
	run_app="jamvm -jar \$app_cmd"
else
	run_app="\$app_file"
fi

run_app="\$run_app \$app_args"

if [ "\$app_daemonize" = "true" ]; then
	daemon="&"
fi

start()
{
	echo -n "Starting \$app_name: "

	mkdir -p /tmp/\$app_name;

	if [ "\$app_daemonize" = "true" ]; then
	    (
		cd \$SBC_USERAPPS/\$app_name; 
		exec > /tmp/\$app_name/stdout; 
		exec 2> /tmp/\$app_name/stderr;
		exec <&-
		\$run_app &
	    ) &
		wait
	else
	    (
		exec > /tmp/\$app_name/stdout; 
		exec 2> /tmp/\$app_name/stderr; 
		exec <&-
		cd \$SBC_USERAPPS/\$app_name; 
		\$run_app
	    )
	fi
}

stop()
{
	ps | grep -v grep | grep "\$run_app" | awk '{print \$1}' | xargs kill -KILL
}

case "\$1" in
  forcestart) echo -n "Starting \$app_name:: "
	start
	echo "OK"
	;;
  start) if [ "\$app_enabled" = "true" ]; then
		echo -n "Starting \$app_name:: "
		start
		echo "OK"
	fi
	;;
  stop) echo -n "Stopping \$app_name:: "
	stop
	echo "OK"
	;;
  restart) echo -n "Restarting \$app_name:: "
	stop
	sleep 1
	start
	echo "OK"
	;;
  *) echo "Usage: \$SCRIPTNAME {start|stop|restart}" >&2
	exit 1
	;;
esac

exit 0
EOF
chmod 755 $SBC_CONFIGDIR/init.d/S$app_priority$1
}
